@@ -43,6 +43,10 @@
 #   include "xap_UnixDlg_HTMLOptions.h"
 #   include "xap_UnixDlg_Password.h"
 #   include "xap_UnixDlg_Image.h"
+#   include "xap_UnixDlg_ListDocuments.h"
+#	include "xap_UnixDlg_History.h"
+#   include "xap_UnixDlg_DocComparison.h"
+
 #	include "ap_UnixDialog_Replace.h"
 #	include "ap_UnixDialog_Break.h"
 #	include "ap_UnixDialog_InsertTable.h"
@@ -53,6 +57,7 @@
 #	include "ap_UnixDialog_Options.h"
 #	include "ap_UnixDialog_Spell.h"
 #	include "ap_UnixDialog_Styles.h"
+#   include "ap_UnixDialog_Stylist.h"
 #	include "ap_UnixDialog_Tab.h"
 #	include "ap_UnixDialog_Insert_DateTime.h"
 #	include "ap_UnixDialog_WordCount.h"
@@ -70,8 +75,11 @@
 #   include "ap_UnixDialog_MarkRevisions.h"
 #   include "ap_UnixDialog_ListRevisions.h"
 #   include "ap_UnixDialog_MergeCells.h"
+#   include "ap_UnixDialog_SplitCells.h"
 #   include "ap_UnixDialog_FormatTable.h"
+#   include "ap_UnixDialog_FormatFrame.h"
 #   include "ap_UnixDialog_FormatFootnotes.h"
+#   include "ap_UnixDialog_FormatTOC.h"
 #   include "ap_UnixDialog_MailMerge.h"
 #if 0
 #	include "ap_UnixDialog_Download_File.h"
@@ -121,6 +129,12 @@
     DeclareDialog(XAP_DIALOG_ID_INSERT_FILE,    XAP_UnixDialog_FileOpenSaveAs)
     DeclareDialog(XAP_DIALOG_ID_PASSWORD,       XAP_UnixDialog_Password)
     DeclareDialog(XAP_DIALOG_ID_IMAGE,          XAP_UnixDialog_Image)
+	DeclareDialog(XAP_DIALOG_ID_LISTDOCUMENTS,  XAP_UnixDialog_ListDocuments)
+	DeclareDialog(XAP_DIALOG_ID_COMPAREDOCUMENTS,XAP_UnixDialog_ListDocuments)
+	DeclareDialog(XAP_DIALOG_ID_MERGEDOCUMENTS, XAP_UnixDialog_ListDocuments)
+    DeclareDialog(XAP_DIALOG_ID_HISTORY,        XAP_UnixDialog_History)
+    DeclareDialog(XAP_DIALOG_ID_DOCCOMPARISON,  XAP_UnixDialog_DocComparison)
+
 	DeclareDialog(AP_DIALOG_ID_REPLACE,			AP_UnixDialog_Replace)
 	DeclareDialog(AP_DIALOG_ID_FIND,			AP_UnixDialog_Replace)
     DeclareDialog(AP_DIALOG_ID_HDRFTR,          AP_UnixDialog_HdrFtr)
@@ -129,6 +143,7 @@
 	DeclareDialog(AP_DIALOG_ID_BREAK,			AP_UnixDialog_Break)
 	DeclareDialog(AP_DIALOG_ID_SPELL,			AP_UnixDialog_Spell)
 	DeclareDialog(AP_DIALOG_ID_STYLES,			AP_UnixDialog_Styles)
+		DeclareDialog(AP_DIALOG_ID_STYLIST,			AP_UnixDialog_Stylist)
     DeclareDialog(AP_DIALOG_ID_PAGE_NUMBERS,    AP_UnixDialog_PageNumbers)
 	DeclareDialog(AP_DIALOG_ID_PARAGRAPH,		AP_UnixDialog_Paragraph)
 	DeclareDialog(AP_DIALOG_ID_OPTIONS,			AP_UnixDialog_Options)
@@ -150,8 +165,11 @@
      DeclareDialog(AP_DIALOG_ID_LIST_REVISIONS,		AP_UnixDialog_ListRevisions)
      DeclareDialog(AP_DIALOG_ID_INSERT_TABLE,		AP_UnixDialog_InsertTable)
      DeclareDialog(AP_DIALOG_ID_MERGE_CELLS,		AP_UnixDialog_MergeCells)
+     DeclareDialog(AP_DIALOG_ID_SPLIT_CELLS,		AP_UnixDialog_SplitCells)
 	 DeclareDialog(AP_DIALOG_ID_FORMAT_TABLE,		AP_UnixDialog_FormatTable)
+	 DeclareDialog(AP_DIALOG_ID_FORMAT_FRAME,		AP_UnixDialog_FormatFrame)
 	 DeclareDialog(AP_DIALOG_ID_FORMAT_FOOTNOTES,		AP_UnixDialog_FormatFootnotes)
+	 DeclareDialog(AP_DIALOG_ID_FORMAT_TOC,		AP_UnixDialog_FormatTOC)
 	 DeclareDialog(AP_DIALOG_ID_MAILMERGE,		AP_UnixDialog_MailMerge)
 #if 0
 	DeclareDialog(AP_DIALOG_ID_DOWNLOAD_FILE,	AP_UnixDialog_Download_File)
