@@ -105,8 +105,8 @@ void AP_Win32Dialog_Options::runModal(XAP_Frame * pFrame)
 	XAP_Win32App * pWin32App = static_cast<XAP_Win32App *>(m_pApp);	
 	AP_Win32Dialog_Options_Sheet	sheet;	
 	
+	UT_return_if_fail (pFrame);	
 	m_pFrame = pFrame;
-	UT_ASSERT(pFrame);	
 	
 	/* Create the property sheet and associate its pages*/		
 	m_toolbars.setContainer(this);	
@@ -195,7 +195,7 @@ HWND	AP_Win32Dialog_Options::getPage(PSH_PAGES page)
 			break;
 	}
 	
-	UT_ASSERT(hWnd!=NULL);
+	UT_ASSERT_HARMLESS(hWnd!=NULL);
 	
 	return hWnd;
 }
@@ -235,10 +235,6 @@ void AP_Win32Dialog_Options::_controlEnable( tControl id, bool value )
 		EnableWindow(GetDlgItem((HWND)getPage(PG_SPELL),AP_RID_DIALOG_OPTIONS_CHK_SpellNumbers),value);
 		return;
 
-	case id_CHECK_SPELL_INTERNET:
-		EnableWindow(GetDlgItem((HWND)getPage(PG_SPELL),AP_RID_DIALOG_OPTIONS_CHK_SpellInternet),value);
-		return;
-
 	case id_CHECK_VIEW_SHOW_RULER:
 		EnableWindow(GetDlgItem((HWND)getPage(PG_LAYOUT),AP_RID_DIALOG_OPTIONS_CHK_ViewShowRuler),value);
 		return;
@@ -275,10 +271,6 @@ void AP_Win32Dialog_Options::_controlEnable( tControl id, bool value )
 		EnableWindow(GetDlgItem((HWND)getPage(PG_PREF),AP_RID_DIALOG_OPTIONS_CHK_ShowSplash),value);
 		return;
 
-	case id_CHECK_OTHER_SAVE_CONTEXT_GLYPHS:
-		EnableWindow(GetDlgItem((HWND)getPage(PG_LANG),AP_RID_DIALOG_OPTIONS_CHK_OtherSaveContextGlyphs),value);
-		return;
-
 	case id_CHECK_OTHER_HEBREW_CONTEXT_GLYPHS:
 		EnableWindow(GetDlgItem((HWND)getPage(PG_LANG),AP_RID_DIALOG_OPTIONS_CHK_OtherHebrewContextGlyphs),value);
 		return;
@@ -324,7 +316,6 @@ DEFINE2_GET_SET_BOOL(PG_SPELL,SpellSuggest);
 DEFINE2_GET_SET_BOOL(PG_SPELL,SpellMainOnly);
 DEFINE2_GET_SET_BOOL(PG_SPELL,SpellUppercase);
 DEFINE2_GET_SET_BOOL(PG_SPELL,SpellNumbers);
-DEFINE2_GET_SET_BOOL(PG_SPELL,SpellInternet);
 
 DEFINE2_GET_SET_BOOL(PG_LAYOUT,ViewShowRuler);
 DEFINE2_GET_SET_BOOL(PG_LAYOUT,ViewShowStatusBar);
@@ -337,7 +328,6 @@ DEFINE2_GET_SET_BOOL(PG_LAYOUT,SmartQuotesEnable);
 DEFINE2_GET_SET_BOOL(PG_PREF,PrefsAutoSave);
 DEFINE2_GET_SET_BOOL(PG_PREF,ShowSplash);
 DEFINE2_GET_SET_BOOL(PG_LANG,OtherDirectionRtl);
-DEFINE2_GET_SET_BOOL(PG_LANG,OtherSaveContextGlyphs);
 DEFINE2_GET_SET_BOOL(PG_LANG,OtherHebrewContextGlyphs);
 DEFINE2_GET_SET_BOOL(PG_LANG,LanguageWithKeyboard);
 DEFINE2_GET_SET_BOOL(PG_LANG,DirMarkerAfterClosingParenthesis);
@@ -707,7 +697,7 @@ void AP_Win32Dialog_Options_Toolbars::_onInitDialog()
 	{		
 		utf8 = static_cast<const char *>(reinterpret_cast<const UT_UTF8String*>(vec.getNthItem(i))->utf8_str());		
 		static char szText[255];		
-		strcpy(szText, (AP_Win32App::s_fromUTF8ToAnsi(utf8)).c_str());		
+		strcpy(szText, (AP_Win32App::s_fromUTF8ToWinLocale(utf8)).c_str());		
 		
 		LVITEM item;
 		item.pszText = szText;
@@ -757,20 +747,6 @@ void AP_Win32Dialog_Options_Spelling::_onCommand(HWND hWnd, WPARAM wParam, LPARA
 		case AP_RID_DIALOG_OPTIONS_CHK_SpellMainOnly:		pParent->_enableDisableLogic(AP_Dialog_Options::id_CHECK_SPELL_MAIN_ONLY);		return ;
 		case AP_RID_DIALOG_OPTIONS_CHK_SpellUppercase:		pParent->_enableDisableLogic(AP_Dialog_Options::id_CHECK_SPELL_UPPERCASE);		return ;
 		case AP_RID_DIALOG_OPTIONS_CHK_SpellNumbers:		pParent->_enableDisableLogic(AP_Dialog_Options::id_CHECK_SPELL_NUMBERS);		return ;
-		case AP_RID_DIALOG_OPTIONS_CHK_SpellInternet:		pParent->_enableDisableLogic(AP_Dialog_Options::id_CHECK_SPELL_INTERNET);		return ;
-		case AP_RID_DIALOG_OPTIONS_COMBO_CUSTOMDICT:
-			return;
-		case AP_RID_DIALOG_OPTIONS_BTN_CUSTOMDICT:
-			UT_DEBUGMSG(("WM_Command for BtnCustomDict\n"));
-			return;
-		case AP_RID_DIALOG_OPTIONS_BTN_IGNOREDRESET:
-			//_event_IgnoreReset();
-			UT_DEBUGMSG(("WM_Command for BtnIgnoreReset\n"));
-			return;
-		case AP_RID_DIALOG_OPTIONS_BTN_IGNOREDEDIT:
-			UT_DEBUGMSG(("WM_Command for BtnIgnoreEdit\n"));
-			return;		
-		
 		default:
 		break;
 	}
@@ -792,7 +768,6 @@ void AP_Win32Dialog_Options_Spelling::_onInitDialog()
 	_DS2(OPTIONS_FRM_SpellIgnore,			DLG_Options_Label_Ignore);
 	_DS2(OPTIONS_CHK_SpellUppercase, 		DLG_Options_Label_SpellUppercase);
 	_DS2(OPTIONS_CHK_SpellNumbers,			DLG_Options_Label_SpellNumbers);
-	_DS2(OPTIONS_CHK_SpellInternet,			DLG_Options_Label_SpellInternet);
 	_DS2(OPTIONS_LBL_CUSTOMDICT, 			DLG_Options_Label_SpellCustomDict);
 	_DS2(OPTIONS_BTN_CUSTOMDICT, 			DLG_Options_Btn_CustomDict);
 	_DS2(OPTIONS_LBL_IGNOREDWORD,			DLG_Options_Label_SpellIgnoredWord);
@@ -822,7 +797,7 @@ AP_Win32Dialog_Options_Lang::~AP_Win32Dialog_Options_Lang()
 	if (m_pVecUILangs)
 	{		
 		for (UT_uint32 i=0; i < m_pVecUILangs->getItemCount(); i++)
-			free (static_cast<void *>(m_pVecUILangs->getNthItem(i)));
+			free ((void *)m_pVecUILangs->getNthItem(i));
 			
 		delete m_pVecUILangs;		
 	}	
@@ -858,7 +833,6 @@ void AP_Win32Dialog_Options_Lang::_onInitDialog()
 
 	_DS2(OPTIONS_FRM_BidiOptions,			DLG_Options_Label_BiDiOptions);
 	_DS2(OPTIONS_CHK_OtherDirectionRtl,		DLG_Options_Label_DirectionRtl);
-	_DS2(OPTIONS_CHK_OtherSaveContextGlyphs, DLG_Options_Label_SaveContextGlyphs);
 	_DS2(OPTIONS_CHK_OtherHebrewContextGlyphs, DLG_Options_Label_HebrewContextGlyphs);
 	
 	_DSX2(OPTIONS_CHK_LanguageWithKeyboard,		DLG_Options_Label_LangWithKeyboard);
@@ -951,7 +925,7 @@ void AP_Win32Dialog_Options_Layout::_onCommand(HWND hWnd, WPARAM wParam, LPARAM
 		case AP_RID_DIALOG_OPTIONS_BTN_BGColor:
 		{
 			pColorDialog = (AP_Dialog_Background *)(pParent->getDialogFactory()->requestDialog(AP_DIALOG_ID_BACKGROUND));
-			UT_ASSERT(pColorDialog);
+			UT_return_if_fail (pColorDialog);
 	
 			UT_parseColor(pParent->_gatherColorForTransparent(), rgbColor );
 			pColorDialog->setColor(rgbColor);
@@ -1040,7 +1014,6 @@ void AP_Win32Dialog_Options_Pref::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lP
 		pParent->_enableDisableLogic(AP_Dialog_Options::id_CHECK_OTHER_DEFAULT_DIRECTION_RTL);	
 		return;
 
-	case AP_RID_DIALOG_OPTIONS_CHK_OtherSaveContextGlyphs:	pParent->_enableDisableLogic(AP_Dialog_Options::id_CHECK_OTHER_SAVE_CONTEXT_GLYPHS);return;
 	case AP_RID_DIALOG_OPTIONS_CHK_OtherHebrewContextGlyphs:  pParent->_enableDisableLogic(AP_Dialog_Options::id_CHECK_OTHER_HEBREW_CONTEXT_GLYPHS);return;
 	case AP_RID_DIALOG_OPTIONS_CHK_AutoSaveFile:
 		pParent->_enableDisableLogic(AP_Dialog_Options::id_CHECK_AUTO_SAVE_FILE);
