@@ -24,11 +24,13 @@
 #include "ut_misc.h"
 #include "ut_units.h"
 #include "ut_vector.h"
+#include "ut_endian.h"
 #include "pt_Types.h"
 #include "ie_exp_RTF.h"
 #include "pd_Document.h"
 #include "pp_AttrProp.h"
 #include "pp_Property.h"
+#include "pp_Revision.h"
 #include "px_ChangeRecord.h"
 #include "px_CR_Object.h"
 #include "px_CR_Span.h"
@@ -60,7 +62,7 @@ IE_Exp_RTF::IE_Exp_RTF(PD_Document * pDocument)
 	m_braceLevel = 0;
 	m_bLastWasKeyword = false;
 	m_atticFormat = false;
-	m_CharRTL = FRIBIDI_TYPE_UNSET;
+	m_CharRTL = UT_BIDI_UNSET;
 }
 
 IE_Exp_RTF::IE_Exp_RTF(PD_Document * pDocument,bool atticFormat)
@@ -72,7 +74,7 @@ IE_Exp_RTF::IE_Exp_RTF(PD_Document * pDocument,bool atticFormat)
 	m_braceLevel = 0;
 	m_bLastWasKeyword = false;
 	m_atticFormat = atticFormat;
-	m_CharRTL = FRIBIDI_TYPE_UNSET;
+	m_CharRTL = UT_BIDI_UNSET;
 }
 
 IE_Exp_RTF::~IE_Exp_RTF()
@@ -86,7 +88,7 @@ IE_Exp_RTF::~IE_Exp_RTF()
 /*****************************************************************/
 
 IE_Exp_RTF_Sniffer::IE_Exp_RTF_Sniffer ()
-	: IE_ExpSniffer(IE_IMPEXPNAME_RTF)
+	: IE_ExpSniffer(IE_IMPEXPNAME_RTF, true)
 {
 	// 
 }
@@ -126,7 +128,7 @@ bool IE_Exp_RTF_Sniffer::getDlgLabels(const char ** pszDesc,
 /*for attic*/
 
 IE_Exp_RTF_attic_Sniffer::IE_Exp_RTF_attic_Sniffer ()
-	: IE_ExpSniffer(IE_IMPEXPNAME_RTFATTIC)
+	: IE_ExpSniffer(IE_IMPEXPNAME_RTFATTIC, false)
 {
 	// 
 }
@@ -259,9 +261,11 @@ void IE_Exp_RTF::exportHdrFtr(const char * pszHdrFtr , const char * pszHdrFtrID,
 
 // First find the header/footer section and id in the document.
 	m_pListenerWriteDoc->_closeSpan();
+#if 0 //#TF
 	m_pListenerWriteDoc->_closeBlock();
 	m_pListenerWriteDoc->_closeSpan();
 	m_pListenerWriteDoc->_closeSection();
+#endif
 	m_pListenerWriteDoc->_setTabEaten(false);
 
 	PL_StruxDocHandle hdrSDH = getDoc()->findHdrFtrStrux((const XML_Char *) pszHdrFtr,(const XML_Char * ) pszHdrFtrID);
@@ -298,14 +302,16 @@ void IE_Exp_RTF::exportHdrFtr(const char * pszHdrFtr , const char * pszHdrFtrID,
 	_rtf_keyword(pszKeyWord);
 	_rtf_keyword("pard");
 	_rtf_keyword("plain");
-	m_pListenerWriteDoc->m_bBlankLine = false;
+	m_pListenerWriteDoc->m_bBlankLine = true;
 	m_pListenerWriteDoc->m_bStartedList = false;
 	UT_DEBUGMSG(("SEVIOR: Doing header \n"));
 //
 // Now pump out the contents of the HdrFtr
 //
 	getDoc()->tellListenerSubset(static_cast<PL_Listener *>(m_pListenerWriteDoc),pExportHdrFtr);
+#if 0 //#TF
 	_rtf_keyword("par");
+#endif
 	delete pExportHdrFtr;
 	_rtf_close_brace();
 }
@@ -539,7 +545,11 @@ void IE_Exp_RTF::_rtf_chardata(const char * pbuf, UT_uint32 buflen)
 	const char * current = pbuf;
 	UT_uint32 count = 0;
 
-	UT_DEBUGMSG(("data = %s\n", pbuf));
+	xxx_UT_DEBUGMSG(("Buffer length = %d \n",buflen));
+	if(buflen < 400)
+	{ 
+		xxx_UT_DEBUGMSG(("data = %s\n", pbuf));
+	}
 	if (m_bLastWasKeyword)
 	{
 		write(" ");
@@ -551,9 +561,9 @@ void IE_Exp_RTF::_rtf_chardata(const char * pbuf, UT_uint32 buflen)
 	}
 
 	conv = UT_iconv_open("UCS-4", "utf-8");
-	UT_ASSERT(conv);
+	UT_return_if_fail (conv);
 	while (count < buflen) {
-		if (*current > 127) { 
+		if (*current & 0x80) {  // check for non-ASCII value
 			UT_UCS4Char wc;
 			size_t insz, sz;
 			char * dest = (char*)(&wc);
@@ -561,12 +571,19 @@ void IE_Exp_RTF::_rtf_chardata(const char * pbuf, UT_uint32 buflen)
 			sz = sizeof(wc);
 			UT_iconv(conv, &current, &insz, &dest, &sz);
 			if (wc > 0x00ff) {
-				UT_ASSERT(UT_NOT_IMPLEMENTED);
+//				UT_ASSERT_HARMLESS(UT_NOT_IMPLEMENTED);
 			}
 			else {
 				_rtf_nonascii_hex2(wc);
 			}
-			count += buflen - insz;
+			if(buflen - insz > 0)
+			{
+				count += buflen - insz;
+			}
+			else
+			{
+				count++;
+			}
 		}
 		else {
 			write (current, 1);
@@ -599,7 +616,7 @@ bool IE_Exp_RTF::_write_rtf_header(void)
 	if (langcode)
 	{
 		const char* cpgname = wvLIDToCodePageConverter(langcode);
-		UT_DEBUGMSG(("Belcon,after wvLIDToCodePageConverter(%d),cpgname=%s\n",langcode,cpgname));
+		xxx_UT_DEBUGMSG(("Belcon,after wvLIDToCodePageConverter(%d),cpgname=%s\n",langcode,cpgname));
 		if (UT_strnicmp(cpgname,"cp",2)==0 && UT_UCS4_isdigit(cpgname[2]))
 		{
 			int cpg;
@@ -624,7 +641,7 @@ bool IE_Exp_RTF::_write_rtf_header(void)
 					wrote_cpg = 1;
 				}
 			}
-			UT_DEBUGMSG(("Belcon:after XAP_EncodingManager::get_instance()->CodepageFromCharset(%s),codepage=%s\n",cpgname,codepage));
+			xxx_UT_DEBUGMSG(("Belcon:after XAP_EncodingManager::get_instance()->CodepageFromCharset(%s),codepage=%s\n",cpgname,codepage));
 		}
 	};
 	if (!wrote_cpg)
@@ -711,7 +728,7 @@ bool IE_Exp_RTF::_write_rtf_header(void)
 	                                    // document properties
 	const XML_Char * pszFootnoteType = NULL;
 	const PP_AttrProp* pDocAP = getDoc()->getAttrProp();
-	UT_ASSERT(pDocAP);
+	UT_return_val_if_fail (pDocAP, false);
 	pDocAP->getProperty("document-footnote-type", (const XML_Char *&)pszFootnoteType);
 	if (pszFootnoteType == NULL)
 	{
@@ -919,6 +936,63 @@ bool IE_Exp_RTF::_write_rtf_header(void)
 //
 	_rtf_keyword("facingp"); // Allow odd-even headers/footers
 	_rtf_keyword("titlepg"); // Allow first page headers/footers
+
+	// revisions stuff
+	const UT_GenericVector<AD_Revision*> & Revs = getDoc()->getRevisions();
+
+	if(Revs.getItemCount())
+	{
+		_rtf_open_brace();
+		_rtf_keyword("*");
+		_rtf_keyword("revtbl");
+		
+		UT_UTF8String s;
+		UT_UCS4String s4;
+
+		// MS Word assumes that the table always starts with author Unknown and it will
+		// not bother to lookup the 0-th author, so we have to insert that dummy entry
+		_rtf_open_brace();
+		_rtf_chardata("Unknown", 7);
+		_rtf_semi();
+		_rtf_close_brace();
+		
+		for(UT_sint32 i = 0; i < Revs.getItemCount(); ++i)
+		{
+			AD_Revision* pRev = Revs.getNthItem(i);
+			if(!pRev)
+			{
+				UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
+				continue;
+			}
+
+			s4 = pRev->getDescription();
+
+			// construct author name from our numerical id and comment
+			// (the id guarantees us uniqueness)
+			UT_UTF8String_sprintf(s, "rev %d (%s)",pRev->getId(),s4.utf8_str());
+			_rtf_open_brace();
+			_rtf_chardata(s.utf8_str(),s.byteLength());
+			_rtf_semi();
+			_rtf_close_brace();
+		}
+		
+		_rtf_close_brace();
+	}
+
+	// underline newly inserted text
+	UT_uint32 iRevMode = 0;
+	if(getDoc()->isShowRevisions())
+	{
+		iRevMode = 3;
+	}
+	
+	_rtf_keyword("revprop", iRevMode);
+	
+	if(getDoc()->isMarkRevisions())
+	{
+		_rtf_keyword("revisions");
+	}
+	
 	return (m_error == 0);
 }
 
@@ -1081,20 +1155,72 @@ void IE_Exp_RTF::_write_tabdef(const char * szTabStops)
 	}
 }
 
+/*!
+ * Get style
+ */
+const XML_Char * IE_Exp_RTF::_getStyleProp(
+	s_RTF_AttrPropAdapter_Style * pADStyle,
+	const s_RTF_AttrPropAdapter * apa,
+	const char * szProp)
+{
+	const XML_Char *szVal = NULL;
+	if(pADStyle != NULL)
+	{
+		szVal = pADStyle->getProperty(szProp);
+		if(szVal == NULL)
+		{
+			szVal = apa->getProperty(szProp);
+		}
+		else
+		{
+			szVal = NULL;
+		}
+	}
+	else
+	{
+		szVal = apa->getProperty(szProp);
+	}
+	return szVal;
+}
+							   
 /*!
  * Write out the <charfmt> paragraph or character formatting. This
  * does not print opening and closing braces.
  */
 void IE_Exp_RTF::_write_charfmt(const s_RTF_AttrPropAdapter & apa)
 {
-	const XML_Char * szColor = apa.getProperty("color");
-	UT_sint32 ndxColor = _findColor((char*)szColor);
-	UT_ASSERT(ndxColor != -1);
+	//const XML_Char * szStyle = apa.getAttribute(PT_STYLE_ATTRIBUTE_NAME);
+	//UT_sint32 iStyle = -1;
+	s_RTF_AttrPropAdapter_Style * pADStyle = NULL;
+#if 0
+	if(szStyle != NULL)
+	{
+		PD_Style * pStyle = NULL;
+		iStyle = static_cast<UT_sint32>(_getStyleNumber(szStyle));
+		getDoc()->getStyle(szStyle,&pStyle);
+		pADStyle = new s_RTF_AttrPropAdapter_Style(pStyle);
+//
+// OK now we have to make sure all these character props aren't in the style
+//
+	}
+#endif
+	const XML_Char * szColor = _getStyleProp(pADStyle,&apa,"color");
 
-	if (ndxColor != 0) // black text, the default
-		_rtf_keyword("cf",ndxColor);
+	UT_sint32 ndxColor = -1;
+	if(szColor)
+	{
+		UT_sint32 ndxColor = _findColor((char*)szColor);
+		if( ndxColor == -1)
+		{
+			return;
+		}
+		UT_return_if_fail (ndxColor != -1);
 
-	szColor = apa.getProperty("bgcolor");
+		if (ndxColor != 0) // black text, the default
+			_rtf_keyword("cf",ndxColor);
+	}
+
+	szColor = _getStyleProp(pADStyle,&apa,"bgcolor");
 
 	if (szColor && UT_stricmp (szColor, "transparent") != 0)
 	{
@@ -1106,12 +1232,19 @@ void IE_Exp_RTF::_write_charfmt(const s_RTF_AttrPropAdapter & apa)
 			_rtf_keyword("highlight",ndxColor);
 		}
 	}
+	const XML_Char * szFont = NULL;
+	if(pADStyle != NULL)
+	{
+		szFont = pADStyle->getProperty("font-family");
+	}
+	if(szFont == NULL)
+	{
+		UT_sint32 ndxFont = _findFont(&apa);
+		if(ndxFont != -1)
+			_rtf_keyword("f",ndxFont);	// font index in fonttbl
+	}
 
-	UT_sint32 ndxFont = _findFont(&apa);
-	if(ndxFont != -1)
-		_rtf_keyword("f",ndxFont);	// font index in fonttbl
-
-	const XML_Char * szFontSize = apa.getProperty("font-size");
+	const XML_Char * szFontSize = _getStyleProp(pADStyle,&apa,"font-size");
 	double dbl = UT_convertToPoints(szFontSize);
 	UT_sint32 d = (UT_sint32)(dbl*2.0);
 
@@ -1122,15 +1255,15 @@ void IE_Exp_RTF::_write_charfmt(const s_RTF_AttrPropAdapter & apa)
 			d = 24;
 		_rtf_keyword("fs",d);	// font size in half points
 	}
-	const XML_Char * szFontStyle = apa.getProperty("font-style");
+	const XML_Char * szFontStyle = _getStyleProp(pADStyle,&apa,"font-style");
 	if (szFontStyle && *szFontStyle && (UT_strcmp(szFontStyle,"italic")==0))
 		_rtf_keyword("i");
 
-	const XML_Char * szFontWeight = apa.getProperty("font-weight");
+	const XML_Char * szFontWeight = _getStyleProp(pADStyle,&apa,"font-weight");
 	if (szFontWeight && *szFontWeight && (UT_strcmp(szFontWeight,"bold")==0))
 		_rtf_keyword("b");
 
-	const XML_Char * szFontDecoration = apa.getProperty("text-decoration");
+	const XML_Char * szFontDecoration = _getStyleProp(pADStyle,&apa,"text-decoration");
 	if (szFontDecoration && *szFontDecoration)
 	{
 		if (strstr(szFontDecoration,"underline") != 0)
@@ -1149,7 +1282,7 @@ void IE_Exp_RTF::_write_charfmt(const s_RTF_AttrPropAdapter & apa)
 		}
 	}
 
-	const XML_Char * szFontPosition = apa.getProperty("text-position");
+	const XML_Char * szFontPosition = _getStyleProp(pADStyle,&apa,"text-position");
 	if (szFontPosition && *szFontPosition)
 	{
 		if (!UT_strcmp(szFontPosition,"superscript"))
@@ -1159,15 +1292,15 @@ void IE_Exp_RTF::_write_charfmt(const s_RTF_AttrPropAdapter & apa)
 	}
 
 	// export the language of the run of text
-	const XML_Char * szLang = apa.getProperty("lang");
+	const XML_Char * szLang = _getStyleProp(pADStyle,&apa,"lang");
 	if ( szLang )
 	  {
-	    UT_DEBUGMSG(("DOM: lang,lid = %s,%d\n", szLang, wvLangToLIDConverter(szLang)));
+	    xxx_UT_DEBUGMSG(("DOM: lang,lid = %s,%d\n", szLang, wvLangToLIDConverter(szLang)));
 	    _rtf_keyword("lang", wvLangToLIDConverter(szLang));
 	  }
 
 	//###TF const XML_Char * szDir = apa.getProperty("dir");
-	const XML_Char * szDirOvrr = apa.getProperty("dir-override");
+	const XML_Char * szDirOvrr = _getStyleProp(pADStyle,&apa,"dir-override");
 
 	bool bProceed = true;
 	if (szDirOvrr)
@@ -1175,13 +1308,13 @@ void IE_Exp_RTF::_write_charfmt(const s_RTF_AttrPropAdapter & apa)
 		if (!UT_strcmp (szDirOvrr, "ltr"))
 		{
 			_rtf_keyword ("ltrch");
-			m_CharRTL = FRIBIDI_TYPE_LTR;
+			m_CharRTL = UT_BIDI_LTR;
 			bProceed = false;
 		}
 		else if (!UT_strcmp (szDirOvrr, "rtl"))
 		{
 			_rtf_keyword ("rtlch");
-			m_CharRTL = FRIBIDI_TYPE_RTL;
+			m_CharRTL = UT_BIDI_RTL;
 			bProceed  = false;
 		}
 	}
@@ -1194,6 +1327,13 @@ void IE_Exp_RTF::_write_charfmt(const s_RTF_AttrPropAdapter & apa)
 			_rtf_keyword ("rtlch");
 	}  */
 
+	const XML_Char * szHidden = _getStyleProp(pADStyle,&apa,"display");
+	if(szHidden && *szHidden && !UT_strcmp(szHidden, "none"))
+	{
+		_rtf_keyword ("v");
+	}
+	
+	
 	const XML_Char * szListTag = apa.getProperty("list-tag");
 	if (szListTag && *szListTag)
 	{
@@ -1203,13 +1343,176 @@ void IE_Exp_RTF::_write_charfmt(const s_RTF_AttrPropAdapter & apa)
 		_rtf_keyword("listtag",id);
 		_rtf_close_brace();
 	}
-
+	DELETEP(pADStyle);
 
 	// TODO do something with our font-stretch and font-variant properties
 	// note: we assume that kerning has been turned off at global scope.
 
+	// MUST BE LAST after all other props have been processed
+	_output_revision(apa, false);
 }
 
+void IE_Exp_RTF::_output_revision(const s_RTF_AttrPropAdapter & apa, bool bPara)
+{
+	const XML_Char *szRevisions = apa.getAttribute("revision");
+	if (szRevisions && *szRevisions)
+	{
+		PP_RevisionAttr RA(szRevisions);
+
+		if(!RA.getRevisionsCount())
+		{
+			UT_return_if_fail( UT_SHOULD_NOT_HAPPEN );
+		}
+
+		// for now, we will just dump the lot; later we need to figure out how to deal
+		// with revision conflicts ...
+		for(UT_uint32 i = 0; i < RA.getRevisionsCount(); ++i)
+		{
+			const PP_Revision * pRev = RA.getNthRevision(i); // a revision found in our attribute
+			if(!pRev)
+			{
+				UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
+				continue;
+			}
+
+			UT_uint32 iId = pRev->getId();
+
+			// now we translated the revision id to and index into the revision table of
+			// the document
+			UT_sint32 iIndx = getDoc()->getRevisionIndxFromId(iId);
+			const UT_GenericVector<AD_Revision*> & RevTbl = getDoc()->getRevisions();
+			if(iIndx < 0 || RevTbl.getItemCount() == 0)
+			{
+				UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
+				continue;
+			}
+
+			AD_Revision * pRevTblItem = RevTbl.getNthItem(iIndx);
+			if(!pRevTblItem)
+			{
+				UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
+				continue;
+			}
+			
+			
+			// the revisions in rtf are marked by a peculiar timestamp, which we now need
+			// to construct
+			time_t t = pRevTblItem->getStartTime();
+			struct tm * pT = gmtime(&t);
+
+			// NB: gmtime counts months 0-11, while dttm 1-12
+
+			UT_uint32 iDttm = pT->tm_min | (pT->tm_hour << 6) | (pT->tm_mday << 11) | ((pT->tm_mon + 1) << 16)
+				| (pT->tm_year << 20) | (pT->tm_wday << 29);
+
+#if 0
+			// according to the docs, we are supposed to emit each of the four bytes of
+			// the dttm int as an ascii char, and if > 127 convert to hex; however, Word
+			// emits this as a normal int and so will we -- the code below is disabled
+			// 
+			// Now that we have the int, we need to convert the 4 bytes to ascii string.
+			// We need to output this in little endian order, I think, since win32 is
+			// inherently LE
+			char Dttm[4];
+			const char * pDttm = (const char *) & iDttm;
+			
+#ifdef UT_LITTLE_ENDIAN
+			Dttm[0] = *pDttm;
+			Dttm[1] = *(pDttm + 1);
+			Dttm[2] = *(pDttm + 2);
+			Dttm[3] = *(pDttm + 3);
+#else
+			Dttm[3] = *pDttm;
+			Dttm[2] = *(pDttm + 1);
+			Dttm[1] = *(pDttm + 2);
+			Dttm[0] = *(pDttm + 3);
+#endif
+			UT_UTF8String s;
+
+			for(UT_uint32 j = 0; j < 4; j++)
+			{
+				if(Dttm[i] <= 127)
+				{
+					s += Dttm[i];
+				}
+				else
+				{
+					UT_String s2;
+					_rtf_nonascii_hex2((UT_sint32)Dttm[i], s2);
+					s += s2.c_str();
+				}
+			}
+#endif
+			if(iIndx < 0)
+			{
+				UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
+				continue;
+			}
+
+			bool bRevisedProps = false;
+
+			const char * pAD     = bPara ? "pnrnot": "revised";
+			const char * pADauth = bPara ? "pnrauth" : "revauth";
+			const char * pADdttm = bPara ? "pnrdate" : "revdttm";
+			
+			const char * pDEL     = "deleted";
+			const char * pDELauth = "revauthdel";
+			const char * pDELdttm = "revdttmdel";
+
+			// it seems that block props cannot be changed in rev mode
+			const char * pCHauth = bPara ? NULL : "crauth";
+			const char * pCHdttm = bPara ? NULL : "crdate";
+
+			switch(pRev->getType())
+			{
+				case PP_REVISION_ADDITION_AND_FMT:
+					bRevisedProps = true;
+					// fall through
+				case PP_REVISION_ADDITION:
+					_rtf_keyword(pAD);
+					_rtf_keyword(pADauth, iIndx+1);
+					_rtf_keyword(pADdttm, iDttm);
+					break;
+					
+				case PP_REVISION_DELETION:
+					_rtf_keyword(pDEL);
+					_rtf_keyword(pDELauth, iIndx+1);
+					_rtf_keyword(pDELdttm, iDttm);
+					break;
+					
+				case PP_REVISION_FMT_CHANGE:
+					bRevisedProps = true;
+
+					if(bPara)
+					{
+						break;
+					}
+					
+					_rtf_keyword(pCHauth, iIndx+1);
+					_rtf_keyword(pCHdttm, iDttm);
+					break;
+
+				default:
+					UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
+			}
+
+			if(bRevisedProps)
+			{
+				// need to dump the props that belong to our revision ...
+				// NB: this might be a recursive call, since _output_revision()
+				// gets (among others) called by _write_charfmt()
+				const PP_AttrProp * pSpanAP = pRev;
+				const PP_AttrProp * pBlockAP = NULL;
+				const PP_AttrProp * pSectionAP = NULL;
+				
+				_write_charfmt(s_RTF_AttrPropAdapter_AP(pSpanAP, pBlockAP, pSectionAP, getDoc()));
+			}
+		} // for
+		
+	} // if(pRevisions)
+}
+
+
 /*!
  * Write out the formatting group for one style in the RTF header.
  */
@@ -1296,7 +1599,7 @@ struct NumberedStyle
  */
 void IE_Exp_RTF::_clearStyles()
 {
-    UT_HASH_PURGEDATA(NumberedStyle *, &m_hashStyles, delete);
+	m_hashStyles.purgeData();
 }
 
 #ifdef _MSC_VER	// MSVC++ warns about 'e' : unreferenced local variable
@@ -1317,7 +1620,7 @@ void IE_Exp_RTF::_selectStyles()
     UT_uint32 nStyleNumber = 0;
     const char * szName;
     const PD_Style * pStyle;
-	UT_Vector vecStyles;
+	UT_GenericVector<PD_Style*> vecStyles;
 	getDoc()->getAllUsedStyles(&vecStyles);
     for (i = 0; getDoc()->enumStyles(i, &szName, &pStyle); ++i)
 	{
@@ -1334,29 +1637,25 @@ void IE_Exp_RTF::_selectStyles()
 			if(pns == NULL)
 			{
 				m_hashStyles.insert(szName, new NumberedStyle(pStyle, ++nStyleNumber));
-				UT_TRY
 				{
-					_rtf_font_info fi(static_cast<s_RTF_AttrPropAdapter_Style>(pStyle));
-					if (_findFont(&fi) == -1)
-						_addFont(&fi);
+					_rtf_font_info fi;
+
+					if (fi.init(static_cast<s_RTF_AttrPropAdapter_Style>(pStyle))) {
+						if (_findFont(&fi) == -1)
+							_addFont(&fi);
+					}
 				}
-				UT_CATCH(UT_CATCH_ANY)
-				{
-				}
-				UT_END_CATCH
 //
 // Now do it for the field font as well
 //
-				UT_TRY
 				{
-					_rtf_font_info fi(static_cast<s_RTF_AttrPropAdapter_Style>(pStyle),true);
-					if (_findFont(&fi) == -1)
-						_addFont(&fi);
+					_rtf_font_info fi;
+
+					if (fi.init(static_cast<s_RTF_AttrPropAdapter_Style>(pStyle),true)) {
+						if (_findFont(&fi) == -1)
+							_addFont(&fi);
+					}
 				}
-				UT_CATCH(UT_CATCH_ANY)
-				{
-				}
-				UT_END_CATCH
 			}
 		}
     }
@@ -1407,11 +1706,9 @@ void IE_Exp_RTF::_write_stylesheets(void)
     _rtf_open_brace();
     _rtf_keyword("stylesheet");
 
-    UT_StringPtrMap::UT_Cursor hc(&m_hashStyles);
+    UT_GenericStringMap<NumberedStyle*>::UT_Cursor hc(&m_hashStyles);
     const NumberedStyle * pns;
-    for (pns = reinterpret_cast<const NumberedStyle *>(hc.first());
-		 hc.is_valid();
-		 pns = reinterpret_cast<const NumberedStyle *>(hc.next()))
+    for (pns = hc.first(); hc.is_valid(); pns = hc.next())
 	{
 		const PD_Style * pStyle = pns->pStyle;
 		_rtf_nl();
@@ -1487,7 +1784,7 @@ void IE_Exp_RTF::_write_listtable(void)
 // Found a child of pList97, it must be a multi-level list.
 //
 				{
-					UT_DEBUGMSG(("SEVIOR: Adding %x to multi-level \n",pAuto));
+					xxx_UT_DEBUGMSG(("SEVIOR: Adding %x to multi-level \n",pAuto));
 					m_vecMultiLevel.addItem((void *) new ie_exp_RTF_MsWord97ListMulti(pAuto));
 					bFoundChild = true;
 					break;
@@ -1495,7 +1792,7 @@ void IE_Exp_RTF::_write_listtable(void)
 			}
 			if(!bFoundChild)
 			{
-				UT_DEBUGMSG(("SEVIOR: Adding %x to simple \n",pAuto));
+				xxx_UT_DEBUGMSG(("SEVIOR: Adding %x to simple \n",pAuto));
 				m_vecSimpleList.addItem((void *) new ie_exp_RTF_MsWord97ListSimple(pAuto));
 			}
 		}
@@ -1524,7 +1821,7 @@ void IE_Exp_RTF::_write_listtable(void)
 			if(!bFoundAtPrevLevel)
 			{
 				ie_exp_RTF_MsWord97List * pCur97 = new ie_exp_RTF_MsWord97List(pList97->getAuto());
-				UT_DEBUGMSG(("SEVIOR: Adding NULL level at depth %d \n",depth));
+				xxx_UT_DEBUGMSG(("SEVIOR: Adding NULL level at depth %d \n",depth));
 				pList97->addLevel(depth, pCur97);
 			}
 			else
@@ -1542,7 +1839,7 @@ void IE_Exp_RTF::_write_listtable(void)
 					{
 						bFoundAtPrevLevel = true;
 						ie_exp_RTF_MsWord97List * pCur97 = new ie_exp_RTF_MsWord97List(pAuto);
-						UT_DEBUGMSG(("SEVIOR: Adding level %x at depth %d \n",pCur97,depth));
+						xxx_UT_DEBUGMSG(("SEVIOR: Adding level %x at depth %d \n",pCur97,depth));
 						pList97->addLevel(depth, pCur97);
 					}
 				}
@@ -1550,7 +1847,7 @@ void IE_Exp_RTF::_write_listtable(void)
 			if(!bFoundAtPrevLevel)
 			{
 				ie_exp_RTF_MsWord97List * pCur97 = new ie_exp_RTF_MsWord97List(pList97->getAuto());
-				UT_DEBUGMSG(("SEVIOR: Adding NULL level at depth %d \n",depth));
+				xxx_UT_DEBUGMSG(("SEVIOR: Adding NULL level at depth %d \n",depth));
 				pList97->addLevel(depth, pCur97);
 			}
 
@@ -1746,7 +2043,7 @@ void IE_Exp_RTF::_output_LevelText(fl_AutoNum * pAuto, UT_uint32 iLevel, UT_UCSC
 		_rtf_nonascii_hex2(lenText,tmp);
 		tmp += LevelText;
 		tmp += ";";
-		UT_DEBUGMSG(("SEVIOR: Final level text string is %s \n",tmp.c_str()));
+		xxx_UT_DEBUGMSG(("SEVIOR: Final level text string is %s \n",tmp.c_str()));
 		write(tmp.c_str());
 		_rtf_close_brace();
 		_rtf_open_brace();
@@ -1775,17 +2072,17 @@ void IE_Exp_RTF::_output_LevelText(fl_AutoNum * pAuto, UT_uint32 iLevel, UT_UCSC
  */
 void IE_Exp_RTF::_generate_level_Text(fl_AutoNum * pAuto,UT_String & LevelText,UT_String &LevelNumbers, UT_uint32 & lenText, UT_uint32 & ifoundLevel)
 {
-	UT_DEBUGMSG(("SEVIOR: pAuto %x \n",pAuto));
+	xxx_UT_DEBUGMSG(("SEVIOR: pAuto %x \n",pAuto));
 	if(pAuto)
 	{
-		UT_DEBUGMSG(("SEVIOR: pAuto-getParent() %x \n",pAuto->getParent()));
+		xxx_UT_DEBUGMSG(("SEVIOR: pAuto-getParent() %x \n",pAuto->getParent()));
 	}
 	if(pAuto && (pAuto->getParent() == NULL))
 	{
 		UT_String LeftSide = pAuto->getDelim();
 		UT_String RightSide;
 		_get_LeftRight_Side(LeftSide,RightSide);
-		UT_DEBUGMSG(("SEVIOR: Top - leftside = %s rightside = %s \n",LeftSide.c_str(),RightSide.c_str()));
+		xxx_UT_DEBUGMSG(("SEVIOR: Top - leftside = %s rightside = %s \n",LeftSide.c_str(),RightSide.c_str()));
 		UT_String place;
 		UT_uint32 locPlace = (UT_uint32) LeftSide.size();
 		_rtf_nonascii_hex2(locPlace+1,place);
@@ -1804,7 +2101,7 @@ void IE_Exp_RTF::_generate_level_Text(fl_AutoNum * pAuto,UT_String & LevelText,U
 			LevelText += RightSide;
 		}
 		lenText = LeftSide.size() + RightSide.size() + 1;
-		UT_DEBUGMSG(("SEVIOR: Level %d LevelText %s  \n",ifoundLevel,LevelText.c_str()));
+		xxx_UT_DEBUGMSG(("SEVIOR: Level %d LevelText %s  \n",ifoundLevel,LevelText.c_str()));
 		return;
 	}
 	else if((pAuto != NULL) && ( pAuto->getParent() != NULL))
@@ -1822,7 +2119,7 @@ void IE_Exp_RTF::_generate_level_Text(fl_AutoNum * pAuto,UT_String & LevelText,U
 		{
 			if(RightSide.size() > 0)
 			{
-				UT_DEBUGMSG(("SEVIOR: RightSide =%s last char = %c \n",RightSide.c_str(),RightSide[RightSide.size()-1]));
+				xxx_UT_DEBUGMSG(("SEVIOR: RightSide =%s last char = %c \n",RightSide.c_str(),RightSide[RightSide.size()-1]));
 			}
 			if(RightSide.size()== 0)
 			{
@@ -1844,7 +2141,7 @@ void IE_Exp_RTF::_generate_level_Text(fl_AutoNum * pAuto,UT_String & LevelText,U
 		LevelText += LeftSide;
 		LevelText += str;
 		LevelText += RightSide;
-		UT_DEBUGMSG(("SEVIOR: Level %d LevelText %s  \n",ifoundLevel,LevelText.c_str()));
+		xxx_UT_DEBUGMSG(("SEVIOR: Level %d LevelText %s  \n",ifoundLevel,LevelText.c_str()));
 		return;
 	}
 	else
@@ -1884,12 +2181,12 @@ void IE_Exp_RTF::_generate_level_Text(fl_AutoNum * pAuto,UT_String & LevelText,U
 void IE_Exp_RTF::_get_LeftRight_Side(UT_String & LeftSide, UT_String & RightSide)
 {
 	const char * psz = strstr(LeftSide.c_str(),"%L");
-	UT_DEBUGMSG(("SEVIOR: Substring = %s Total is %s \n",psz,LeftSide.c_str()));
+	xxx_UT_DEBUGMSG(("SEVIOR: Substring = %s Total is %s \n",psz,LeftSide.c_str()));
 	if(psz != NULL)
 	{
 		UT_uint32 index = (UT_uint32) (psz - LeftSide.c_str());
 		UT_uint32 len = (UT_uint32) strlen(LeftSide.c_str());
-		UT_DEBUGMSG(("SEVIOR: index = %d len =%d \n",index,len));
+		xxx_UT_DEBUGMSG(("SEVIOR: index = %d len =%d \n",index,len));
 		if(index+2 < len)
 		{
 			RightSide = LeftSide.substr(index+2,len);
@@ -1922,7 +2219,7 @@ void IE_Exp_RTF::_output_ListRTF(fl_AutoNum * pAuto, UT_uint32 iLevel)
 	UT_sint32 Param = 0;
 	UT_String fontName;
 	UT_UCSChar bulletsym=0;
-	List_Type lType = NUMBERED_LIST;
+	FL_ListType lType = NUMBERED_LIST;
 	if(pAuto != NULL)
 	{
 		lType = pAuto->getType();
@@ -2041,14 +2338,18 @@ void IE_Exp_RTF::_output_ListRTF(fl_AutoNum * pAuto, UT_uint32 iLevel)
 		PL_StruxDocHandle sdh = pAuto->getFirstItem();
 		const char * szIndent = NULL;
 		const char * szAlign = NULL;
+		// TODO -- we have a problem here; props and attrs are, due to revisions, view dependent and
+		// we have no access to the view, so we will assume that revisions are showing and will ask
+		// for the cumulative result of all of them (revision level PD_MAX_REVISION)
+		// 
 		if(sdh != NULL)
 		{
-			bool bres = getDoc()->getPropertyFromSDH(sdh,"text-indent",&szIndent);
+			bool bres = getDoc()->getPropertyFromSDH(sdh,true,PD_MAX_REVISION,"text-indent",&szIndent);
 			if(bres)
 			{
 				_rtf_keyword_ifnotdefault_twips("fi",(char*)szIndent,0);
 			}
-			bres = getDoc()->getPropertyFromSDH(sdh,"margin-left",&szAlign);
+			bres = getDoc()->getPropertyFromSDH(sdh,true,PD_MAX_REVISION,"margin-left",&szAlign);
 			if(bres)
 			{
 				_rtf_keyword_ifnotdefault_twips("li",(char*)szAlign,0);
@@ -2064,14 +2365,19 @@ void IE_Exp_RTF::_output_ListRTF(fl_AutoNum * pAuto, UT_uint32 iLevel)
 //
 	if(Param == 23)
 	{
-		_rtf_font_info fi(fontName.c_str());
-		UT_sint32 ifont = _findFont(&fi);
-		if(ifont < 0)
-		{
-		  UT_ASSERT_NOT_REACHED();
-			ifont = 0;
-		}
-		_rtf_keyword("f",ifont);
+			{
+				_rtf_font_info fi;
+
+				if (fi.init(fontName.c_str())) {
+					UT_sint32 ifont = _findFont(&fi);
+					if(ifont < 0)
+					{
+						UT_ASSERT_NOT_REACHED();
+						ifont = 0;
+					}
+					_rtf_keyword("f",ifont);
+				}
+			}
 	}
 }
 
@@ -2162,17 +2468,15 @@ UT_sint32 IE_Exp_RTF::_findFont(const _rtf_font_info * pfi) const
 UT_sint32 IE_Exp_RTF::_findFont(const s_RTF_AttrPropAdapter * apa) const
 {
 	static UT_sint32 ifont = 0;
-	UT_TRY
-	  {
-	    _rtf_font_info fi(*apa);
-	    ifont = _findFont(&fi);
-	    return ifont;
-	  }
-	UT_CATCH(UT_CATCH_ANY)
-	  {
-	    return -1;
-	  }
-	UT_END_CATCH
+
+	_rtf_font_info fi;
+	
+	if (fi.init(*apa)) {
+		ifont = _findFont(&fi);
+		return ifont;
+	}
+	
+	return -1;
 }
 
 /*!
@@ -2182,15 +2486,19 @@ UT_sint32 IE_Exp_RTF::_findFont(const s_RTF_AttrPropAdapter * apa) const
 void IE_Exp_RTF::_addFont(const _rtf_font_info * pfi)
 {
 	UT_return_if_fail(pfi && (_findFont(pfi)==-1));
-	_rtf_font_info * pNew = new _rtf_font_info(*pfi);
+
+	_rtf_font_info * pNew = new _rtf_font_info (*pfi);
+	
 	if (pNew)
 		m_vecFonts.addItem(pNew);
-
-	return;
 }
 
-_rtf_font_info::_rtf_font_info(const s_RTF_AttrPropAdapter & apa, bool bDoFieldFont)
-	UT_THROWS((_rtf_no_font))
+_rtf_font_info::_rtf_font_info()
+{
+	// TODO: set these to some default bogus values
+}
+
+bool _rtf_font_info::init(const s_RTF_AttrPropAdapter & apa, bool bDoFieldFont)
 {
     // Not a typo. The AbiWord "font-family" property is what RTF
     // calls font name. It has values like "Courier New".
@@ -2213,8 +2521,7 @@ _rtf_font_info::_rtf_font_info(const s_RTF_AttrPropAdapter & apa, bool bDoFieldF
 	}
     if (szName == NULL)
 	{
-		_rtf_no_font e;
-		UT_THROW(e);
+		return false;
 	}
 
     static const char * t_ff[] = { "fnil", "froman", "fswiss", "fmodern", "fscript", "fdecor", "ftech", "fbidi" };
@@ -2230,22 +2537,21 @@ _rtf_font_info::_rtf_font_info(const s_RTF_AttrPropAdapter & apa, bool bDoFieldF
     nCharset = XAP_EncodingManager::get_instance()->getWinCharsetCode();
     nPitch = fp;
     fTrueType = tt;
-}
 
+	return true;
+}
 
 _rtf_font_info::~_rtf_font_info(void)
 {
 }
 
-_rtf_font_info::_rtf_font_info(const char * szFontName)
-	UT_THROWS((_rtf_no_font))
+bool _rtf_font_info::init(const char * szFontName)
 {
     // Not a typo. The AbiWord "font-family" property is what RTF
     // calls font name. It has values like "Courier New".
     if (szFontName == NULL)
 	{
-		_rtf_no_font e;
-		UT_THROW(e);
+		return false;
 	}
 
 	m_szName = szFontName;
@@ -2263,6 +2569,8 @@ _rtf_font_info::_rtf_font_info(const char * szFontName)
     nCharset = XAP_EncodingManager::get_instance()->getWinCharsetCode();
     nPitch = fp;
     fTrueType = tt;
+
+	return true;
 }
 
 /*!
