@@ -40,6 +40,8 @@ class PD_Document;
 class pf_Frag;
 class UT_Stack;
 
+typedef struct field;
+
 struct bookmark
 {
 	XML_Char * name;
@@ -56,6 +58,27 @@ struct footnote
 	UT_uint32  pid;
 };
 
+
+struct textbox
+{
+	UT_uint32  lid;
+	UT_uint32  ref_pos;
+	UT_uint32  txt_pos;
+	UT_uint32  txt_len;
+	UT_sint32  iLeft;
+	UT_sint32  iWidth;
+	UT_sint32  iTop;
+	UT_sint32  iHeight;
+	UT_sint32  iPosType;
+	UT_sint32  iBorderWidth;
+};
+
+struct textboxPos
+{
+	UT_uint32 lid;
+	pf_Frag * endFrame;
+};
+
 typedef enum
 	{
 		HF_HeaderFirst = 0,
@@ -189,13 +212,17 @@ private:
 	void        _generateParaProps(UT_String &s, const PAP * apap, wvParseStruct *ps);
 	int         _handleBookmarks(const wvParseStruct *ps);
 	void        _handleNotes(const wvParseStruct *ps);
+	void        _handleTextBoxes(const wvParseStruct *ps);
 	bool        _insertNoteIfAppropriate(UT_uint32 iDocPosition,UT_UCS4Char c);
 	bool        _insertFootnote(const footnote * f, UT_UCS4Char c);
 	bool        _insertEndnote(const footnote * f, UT_UCS4Char c);
 	bool        _handleNotesText(UT_uint32 iPos);
+	bool        _handleTextboxesText(UT_uint32 iPos);
+	bool        _findNextTextboxSection();
 	bool        _findNextFNoteSection();
 	bool        _findNextENoteSection();
 	bool        _shouldUseInsert()const;
+	bool        _ensureInBlock();
 	bool        _appendStrux(PTStruxType pts, const XML_Char ** attributes);
 	bool        _appendObject(PTObjectType pto, const XML_Char ** attributes);
 	bool        _appendSpan(const UT_UCSChar * p, UT_uint32 length);
@@ -204,11 +231,17 @@ private:
 	bool        _appendSpanHdrFtr(const UT_UCSChar * p, UT_uint32 length);
 	bool		_appendFmt(const XML_Char ** attributes);
 	void        _handleHeaders(const wvParseStruct *ps);
-	bool        _handleHeadersText(UT_uint32 iPos);
+	bool        _handleHeadersText(UT_uint32 iPos, bool bDoBlockIns);
 	bool        _build_ColumnWidths(UT_NumberVector & colWidths);
 	bool        _isVectorFull(UT_NumberVector & vec);
 	void        setNumberVector(UT_NumberVector & vec, UT_sint32 i, UT_sint32 val);
 	bool        findMatchSpan(UT_sint32 iLeft,UT_sint32 iRight);
+	bool        _ignorePosition(UT_uint32 pos);
+
+	bool        _isTOCsupported(field *f);
+	bool        _insertTOC(field *f);
+	
+	
 	UT_UCS4String		m_pTextRun;
 	//UT_uint32			m_iImageCount;
 	UT_uint32			m_nSections;
@@ -231,7 +264,7 @@ private:
 	bool m_bInPara;
 	bool m_bLTRCharContext;
 	bool m_bLTRParaContext;
-	FriBidiCharType  m_iOverrideIssued;
+	UT_BidiCharType  m_iOverrideIssued;
 	bool m_bBidiMode;
 	bookmark * m_pBookmarks;
 	UT_uint32  m_iBookmarksCount;
@@ -239,6 +272,8 @@ private:
 	UT_uint32  m_iFootnotesCount;
 	footnote * m_pEndnotes;
 	UT_uint32  m_iEndnotesCount;
+	textbox *  m_pTextboxes;
+	UT_uint32  m_iTextboxCount;
 	UT_Vector  m_vLists;
 	UT_uint32  m_iListIdIncrement[9];
 	UT_uint32  m_iMSWordListId;
@@ -252,8 +287,8 @@ private:
 	bool		m_bRowOpen;						// row strux open ?
 	bool		m_bCellOpen;					// cell strux open ?
 	UT_NumberVector	m_vecColumnSpansForCurrentRow;	// placeholder for horizontal cell spans
-	UT_Vector	m_vecColumnWidths;
-	UT_Vector   m_vecEmObjects;               // Objects between cell
+	UT_GenericVector<MsColSpan *>	m_vecColumnWidths;
+	UT_GenericVector<emObject*>   m_vecEmObjects;               // Objects between cell
 											  // struxes
 	UT_NumberVector m_vecColumnPositions;
 	UT_String   m_charProps;
@@ -290,6 +325,17 @@ private:
 	UT_Dimension m_dim;
 	UT_sint32    m_iLeft;
 	UT_sint32    m_iRight;
+	UT_uint32    m_iTextboxesStart;
+	UT_uint32    m_iTextboxesEnd;
+	UT_uint32    m_iNextTextbox;
+	UT_uint32    m_iPrevHeaderPosition;
+	bool         m_bEvenOddHeaders;
+
+	UT_sint32    m_bInTOC;
+	bool         m_bTOCsupported;
+	bool         m_bInTextboxes;
+	pf_Frag *    m_pTextboxEndSection;
+	UT_GenericVector<textboxPos *> m_vecTextboxPos;
 	UT_sint32    m_iLeftCellPos;
 };
 
